/*
 * Programa Proyecto Precipitador Electroesttico
 * ==============================================
 *
 * (c) 2002 Aplik S.A.
 * (c) 2002 Daniel Serpell C.
 *
 * Esta archivo contiene Informacin de propiedad de Aplik S.A., y
 * debe ser tratada como confidencial. Queda prohibida su publicacin
 * o divulgacin.
 *
 */

/*
 * camara.c : Rutinas de procesamiento de imgen.
 */

#include "capture.h"
#include "debug.h"

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <jpeglib.h>
#include <math.h>

#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/time.h>

/* Rutina que escribe un JPG en b/w. debera estar en su propio archivo... */
static int WriteJPEG(char * filename, int sx, int sy, unsigned char *bY)
{
   FILE *of;
   struct jpeg_compress_struct cinfo;
   struct jpeg_error_mgr jerr;
   JSAMPARRAY arrp[3]={0,0,0};
   int i;
   
   /* Punteros a las lneas */
   unsigned char * rY[sy];
   
   /* Redondea el tamao */
   sx = sx & ~15;
   sy = sy & ~15;

   /* Llena los punteros a las lneas */
   for(i=0;i<sy;i++)
      rY[i] = bY + i * sx;
   
   /* Verifica si el archivo ya exista */
   if( (of = fopen( filename, "r" )) )
   {
      fclose(of);
      /* Lo borra */
      unlink( filename );
   }
   
   if( !(of = fopen(filename, "w")) )
   {
      EPRINTF("No puede abrir destino '%s'\n",filename);
      return -1;
   }
   
   cinfo.err = jpeg_std_error(&jerr);  /* Usa el manejador de errores normal */
   jpeg_create_compress(&cinfo);       /* Crea el objeto de compresin JPEG */
   jpeg_stdio_dest(&cinfo, of );       /* El destino es el stream "of" */
   
   /* Parmetros de la imgen a comprimir */
   cinfo.image_width = sx;
   cinfo.image_height = sy;
   cinfo.input_components = 1;
   cinfo.in_color_space = JCS_GRAYSCALE;  
   cinfo.input_gamma = 1.0;
   
   jpeg_set_defaults(&cinfo);          /* Llena el resto de los parmetros */
   
   cinfo.raw_data_in = TRUE;           /* Datos en modo "raw" */
   cinfo.comp_info[0].v_samp_factor = 2;
   cinfo.comp_info[1].v_samp_factor = 2;
   cinfo.comp_info[2].v_samp_factor = 2;
   cinfo.comp_info[0].h_samp_factor = 2;
   cinfo.comp_info[1].h_samp_factor = 1;
   cinfo.comp_info[2].h_samp_factor = 1;
   
   jpeg_start_compress(&cinfo,TRUE);   /* Comienza la compresin */
   
   i = 0;
   while(i<sy)
   {
      arrp[0] = &rY[i];
      i += jpeg_write_raw_data(&cinfo,arrp,sy-i);   /* Enva un bloque de datos */
   }
   
   jpeg_finish_compress(&cinfo);       /* Fin de la compresin */
   fclose(of);
   
   jpeg_destroy_compress(&cinfo);      /* Destruye el objeto */
   return 0;
}

int main(int argc, char **argv)
{
    struct_capture *cp;
    struct capture_options cop;
	unsigned char imagen[640*480];
	int i,fnum=0;

    cop.devname = "/dev/video0";
    cop.sizex = 640;
    cop.sizey = 480;
    cop.format = CAPTURE_FORMAT_GRAY8;
    cop.channel = 1;
    
    cp = capture_init(&cop,0);

    if(!cp)
       return 0;

    capture_getframe(cp, fnum);

    for(i=0;;i++,fnum=1-fnum)
    {
		unsigned char *buf;
		int j,suma;
		char nombre[80];
		capture_getframe(cp, 1 - fnum);	    /* Captura el cuadro siguiente */
		capture_syncframe(cp,fnum);	    /* Espera el cuadro... */
		buf = capture_getbuffer(cp,fnum);   /* Y obtiene un puntero al buffer */
		
		for(suma=0,j=0;j<640*480;j++)
		{
			int d = buf[j] - imagen[j];
			if ( d < 0 ) d = -d;
			suma = suma + d;
		}
		if( suma > 640*480*10 )
		{
			sprintf(nombre,"fotos/imagen-%04d.jpeg",i);
			WriteJPEG(nombre,640,480,buf);
		}
		memcpy( imagen, buf, 640*480 );
    }
	return 0;
}  
